from http import HTTPStatus

from rasax.community.api.decorators import validate_schema, rasa_x_scoped
from sanic import Blueprint, response
from sanic.request import Request

import rasax.community.utils.common as common_utils
from rasax.community.services.assistant_service import (
    AssistantService,
    DuplicateBusinessLogicFlowException,
)


def blueprint() -> Blueprint:
    """Declare endpoints for the assistant.

    Returns:
        Blueprint with endpoints related to the assistant.
    """
    assistant_endpoints = Blueprint("assistant_endpoints")

    @assistant_endpoints.route("/business_logic_flows", methods=["GET"])
    @rasa_x_scoped("assistant.view")
    async def list_business_logic_flows(request: Request) -> response.HTTPResponse:
        """Lists business logic flows.

        Args:
            request: The incoming request.

        Returns:
            Response with a paginated list of business logic flows.
        """
        assistant_service = AssistantService.from_request(request)

        limit = common_utils.int_arg(request, "limit")
        offset = common_utils.int_arg(request, "offset")

        flows, total = assistant_service.fetch_flows(limit=limit, offset=offset)

        return response.json(flows, headers={"X-Total-Count": total})

    @assistant_endpoints.route("/business_logic_flows", methods=["POST"])
    @validate_schema("business_logic_flow")
    @rasa_x_scoped("assistant.modify")
    async def create_business_logic_flow(request: Request) -> response.HTTPResponse:
        """Creates a new business logic flow.

        Args:
            request: The incoming request.

        Returns:
            Response that contains the newly created  flow.
        """
        assistant_service = AssistantService.from_request(request)

        rjs = request.json
        name = rjs.get("name")
        elements = rjs.get("elements")

        try:
            flow = assistant_service.create_flow(name, elements)
            return response.json(
                flow.as_dict(include_elements=True), HTTPStatus.CREATED
            )
        except ValueError as e:
            return common_utils.error(
                HTTPStatus.BAD_REQUEST, "CreateBusinessLogicFlowFailed", details=e,
            )
        except DuplicateBusinessLogicFlowException:
            return common_utils.error(
                HTTPStatus.UNPROCESSABLE_ENTITY, "DuplicateBusinessLogicFlowName"
            )

    return assistant_endpoints
